import { AppBar, Autocomplete, Box, Chip, CircularProgress, Dialog, Divider, Grid, IconButton, Paper, TextField, Toolbar, Typography } from '@mui/material'
import React, { Fragment, useEffect, useState } from 'react'
import { IRegion } from '../interfaces/fuel'
import { useRegions, useServers, useServersInfo } from '../hooks/swrHooks'
import FullFeaturedCrudGrid from './TableEditable'
import ServerService from '../services/ServersService'
import { GridColDef } from '@mui/x-data-grid'
import { Close, Cloud, CloudOff } from '@mui/icons-material'
import ServerData from './ServerData'
import { IServersInfo } from '../interfaces/servers'

export default function ServersView() {
    const [open, setOpen] = useState(false)
    const [options, setOptions] = useState<IRegion[]>([])
    const [search, setSearch] = useState<string | null>("")
    const [debouncedSearch, setDebouncedSearch] = useState<string | null>("")
    const [selectedOption, setSelectedOption] = useState<IRegion | null>(null)
    const { regions, isLoading } = useRegions(10, 1, debouncedSearch)

    const { serversInfo } = useServersInfo(selectedOption?.id)


    const [serverDataOpen, setServerDataOpen] = useState(false)
    const [currentServerData, setCurrentServerData] = useState<any | null>(null)

    useEffect(() => {
        const handler = setTimeout(() => {
            setDebouncedSearch(search)
        }, 500)

        return () => {
            clearTimeout(handler)
        }
    }, [search])

    useEffect(() => {
        if (regions) {
            setOptions([...regions])
        }
    }, [regions])

    const handleInputChange = (value: string) => {
        setSearch(value)
    }

    const handleOptionChange = (value: IRegion | null) => {
        setSelectedOption(value)
    }

    const { servers, isLoading: serversLoading } = useServers(selectedOption?.id, 0, 10)

    const serversColumns: GridColDef[] = [
        //{ field: 'id', headerName: 'ID', type: "number" },
        { field: 'name', headerName: 'Название', type: "string", editable: true },
    ]

    return (
        <>
            <Dialog
                fullScreen
                open={serverDataOpen}
                onClose={() => {
                    setServerDataOpen(false)
                }}
                aria-labelledby="modal-modal-title"
                aria-describedby="modal-modal-description">
                <AppBar sx={{ position: 'sticky' }}>
                    <Toolbar>
                        <IconButton
                            edge="start"
                            color="inherit"
                            onClick={() => {
                                setServerDataOpen(false)
                            }}
                            aria-label="close"
                        >
                            <Close />
                        </IconButton>
                    </Toolbar>
                </AppBar>

                {currentServerData &&
                    <ServerData
                        id={currentServerData?.id}
                        region_id={currentServerData?.region_id}
                        name={currentServerData?.name}
                    />
                }
            </Dialog>

            <Autocomplete
                open={open}
                onOpen={() => {
                    setOpen(true)
                }}
                onClose={() => {
                    setOpen(false)
                }}
                onInputChange={(_, value) => handleInputChange(value)}
                onChange={(_, value) => handleOptionChange(value)}
                filterOptions={(x) => x}
                isOptionEqualToValue={(option: IRegion, value: IRegion) => option.name === value.name}
                getOptionLabel={(option: IRegion) => option.name ? option.name : ""}
                options={options}
                loading={isLoading}
                value={selectedOption}
                renderInput={(params) => (
                    <TextField
                        {...params}
                        label="Район"
                        InputProps={{
                            ...params.InputProps,
                            endAdornment: (
                                <Fragment>
                                    {isLoading ? <CircularProgress color="inherit" size={20} /> : null}
                                    {params.InputProps.endAdornment}
                                </Fragment>
                            )
                        }}
                    />
                )}
            />

            {servers &&
                <Box sx={{ display: 'flex', flexDirection: 'column', gap: '16px', height: '100%' }}>
                    {serversInfo &&
                        <Grid container spacing={{ xs: 2, md: 3 }} columns={{ xs: 1, sm: 1, md: 2, lg: 3, xl: 4 }}>
                            {serversInfo.map((serverInfo: IServersInfo) => (
                                <Grid key={`si-${serverInfo.id}`} item xs={1} sm={1} md={1}>
                                    <Paper sx={{ display: 'flex', flexDirection: 'column', gap: '16px', p: '16px' }}>
                                        <Typography fontWeight={600}>
                                            {serverInfo.name}
                                        </Typography>

                                        <Divider />

                                        <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}>
                                            <Typography>
                                                Количество IP:
                                            </Typography>

                                            <Typography variant="h6" fontWeight={600}>
                                                {serverInfo.IPs_count}
                                            </Typography>
                                        </Box>

                                        <Box sx={{ display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}>
                                            <Typography>
                                                Количество серверов:
                                            </Typography>

                                            <Typography variant="h6" fontWeight={600}>
                                                {serverInfo.servers_count}
                                            </Typography>
                                        </Box>

                                        <Chip
                                            icon={serverInfo.status === "Online" ? <Cloud /> : serverInfo.status === "Offline" ? <CloudOff /> : <CloudOff />}
                                            variant="outlined"
                                            label={serverInfo.status}
                                            color={serverInfo.status === "Online" ? "success" : serverInfo.status === "Offline" ? "error" : "error"}
                                        />
                                    </Paper>
                                </Grid>
                            ))}
                        </Grid>
                    }
                </Box>
            }

            {serversLoading ?
                <CircularProgress />
                :
                servers &&
                <FullFeaturedCrudGrid
                    onSave={(id: any) => {
                        console.log(id)
                    }}
                    onDelete={ServerService.removeServer}
                    initialRows={servers}
                    columns={serversColumns}
                    actions
                    onRowClick={(params, event, details) => {
                        setCurrentServerData(params.row)
                        setServerDataOpen(true)
                    }}
                />
            }
        </>
    )
}